﻿#load @"C:\Users\Mathias\Desktop\Explore\fsharpchart.fsx"

open System
open System.Drawing
open System.Collections.Generic
open System.IO
open System.Linq
open System.Net
open System.Text
open MSDN.FSharp.Charting

let baseUrl = "http://ichart.finance.yahoo.com/table.csv?s="

let fetch (startDate:DateTime) (endDate:DateTime) symbol =
   let query = String.Format("&a={0}&b={1}&c={2}&d={3}&e={4}&f={5}&g=d&ignore=.csv", startDate.Month - 1, startDate.Day, startDate.Year, endDate.Month - 1, endDate.Day, endDate.Year)
   let url = baseUrl + symbol + query
   let request = WebRequest.Create(url)
   use response = request.GetResponse()
   use reader = new StreamReader(response.GetResponseStream(), Encoding.ASCII)
   reader.ReadToEnd()

let cleanup (result:string) = 
   let lines = result.Split((char)10)
   [
      for i in 1 .. lines.Length - 1 do
         yield lines.[i]
   ] 
   |> List.rev 

type Quote = 
   { 
      Symbol:string;
      Date:DateTime;
      Open:float;
      Close:float;
      Low:float;
      High:float;
      Volume:Int64
   }

let retrieveQuotes (symbol:string) (lines:string list) =
   let parse (line:string) = 
      let q = line.Split(',')
      match q with
      | [| d;o;h;l;c;v;_|] -> 
         let quote = 
            { 
               Symbol = symbol;
               Date = DateTime.Parse(d);
               Open = Double.Parse(o);
               High = Double.Parse(h);
               Low = Double.Parse(l);
               Close = Double.Parse(c);
               Volume = Int64.Parse(v)
            }
         Some(quote)
      | _ -> None
   List.map parse lines |> List.choose (fun q -> q)

let read (startDate:DateTime) (endDate:DateTime) symbol =
   fetch startDate endDate symbol
   |> cleanup 
   |> retrieveQuotes symbol

let q = read (new DateTime(2010, 1, 1)) (new DateTime(2011, 2, 1)) "MSFT"
let openValues = List.map (fun i -> (i.Date, i.Open)) q
let closeValues = List.map (fun i -> (i.Date, i.Close)) q
let minValues = List.map (fun i -> (i.Date, i.Low)) q
let maxValues = List.map (fun i -> (i.Date, i.High)) q

FSharpChart.Combine[FSharpChart.Line openValues; FSharpChart.Line closeValues]
FSharpChart.Combine[FSharpChart.Line minValues; FSharpChart.Line maxValues]

Console.ReadLine()